package outils;

import java.util.Iterator;

public class ListeTableau<Type> implements Liste<Type> {
	private int nombreDeParcours = 0;
	private class ParcoursALEndroit implements Parcours<Type> {
		private int indice;
		public ParcoursALEndroit() {
			indice = 0;
		}
		public boolean encore() {
			return indice < nombreDElements();
		}
		public Type elementCourant() {
			return element(indice);
		}
		public void suivant() {
			indice++;
		}
		public void precedent() {
			indice--;
		}
		public void supprime() {
			decaleAGaucheAPartirDe(indice);
			nbElem--;
		}
	}
	private class ParcoursALEnvers implements Parcours<Type> {
		private int indice;
		public ParcoursALEnvers() {
			indice = nombreDElements() - 1;
		}
		public boolean encore() {
			return indice >= 0;
		}
		public Type elementCourant() {
			return element(indice);
		}
		public void suivant() {
			indice--;
		}
		public void precedent() {
			indice++;
		}
		public void supprime() {
			decaleAGaucheAPartirDe(indice);
			nbElem--;
		}
	}
    private Object contenu[];
    private int nbElem;
    public ListeTableau() {
        contenu = new Object[5];
        nbElem = 0;
    }
    public boolean estVide() {
        return nbElem == 0;
    }
    public int nombreDElements() {
        return nbElem;
    }
    public void ajoute(Type element) {
    	if (nbElem == contenu.length) {
    		Object nouveauContenu[] = new Object[2 * contenu.length];
    		for(int i = 0; i < nbElem; i++) nouveauContenu[i] = contenu[i];
    		contenu = nouveauContenu;
    	}
        contenu[nbElem] = element;
        nbElem++;
    }
    private void decaleAGaucheAPartirDe(int index) {
        int indexCourant, indexSuivant;
        for (indexCourant = index, indexSuivant = index + 1; indexCourant < nbElem - 1; indexCourant++, indexSuivant++) {
            contenu[indexCourant] = contenu[indexSuivant];
        }
    }
    public void retire(int index) {
//      if(index < 0 || index >= nbElem) return;
        decaleAGaucheAPartirDe(index);
        nbElem--;
    }
    @SuppressWarnings("unchecked")
	public Type element(int index) {
//        if(index < 0 || index >= nbElem) return null;
        return (Type)(contenu[index]);
    }
    public Parcours<Type> parcours() {
    	nombreDeParcours++;
    	if (nombreDeParcours % 2 == 0)
    		return new ParcoursALEnvers();
    	else
    		return new ParcoursALEndroit();
    }
	public Iterator<Type> iterator() {
		return new AdaptateurDeParcours<Type>(new ParcoursALEndroit());
	}
}
