package temps;

import java.util.StringTokenizer;

/**
 * La classe Date décrit des jours du calendrier.
 *
 * @author Philippe
 * @version 1
 */
public class Date {
    private int leJour, leMois, lAnnee;

    /**
     * Constructeur d'une Date
     */
    public Date() {
        leJour = 1;
        leMois = 1;
        lAnnee = 2025;
    }
    
    /**
     * initialise une Date
     * 
     * @param  j un entier qui précise le jour du mois
     * @param  m un entier qui précise le mois de la date
     * @param  a un entier qui précise l'année de la date
     * @throws MauvaiseValeurDuJour 
     * @throws MauvaiseValeurDuMois 
     * @throws ValeurTropGrandeDuJour 
     */
    private void initialise(int j, int m, int a) throws MauvaiseValeurDuJour, MauvaiseValeurDuMois, ValeurTropGrandeDuJour {
    	lAnnee = a;
    	if (j < 1 || j > 31) throw new MauvaiseValeurDuJour(j);
    	if (m < 1 || m > 12) throw new MauvaiseValeurDuMois(m);
    	if (j > nombreDeJoursDuMois(m, a)) throw new ValeurTropGrandeDuJour(j, m, a);
    	leJour = j;
    	leMois = m;
    }
    
    /**
     * Constructeur d'une Date
     * 
     * @param  j un entier qui précise le jour du mois
     * @param  m un entier qui précise le mois de la date
     * @param  a un entier qui précise l'année de la date
     * @throws ValeurTropGrandeDuJour 
     * @throws MauvaiseValeurDuMois 
     * @throws MauvaiseValeurDuJour 
     */
    public Date(int j, int m, int a) throws MauvaiseValeurDuJour, MauvaiseValeurDuMois, ValeurTropGrandeDuJour {
        initialise(j, m, a);
    }
    
    /**
     * Constructeur d'une Date
     * 
     * @param  date un texte notant la date au format jj/mm/aaaa
     * @throws ValeurTropGrandeDuJour 
     * @throws MauvaiseValeurDuMois 
     * @throws MauvaiseValeurDuJour 
     * @throws MauvaisFormatDeDate 
     */
    public Date(String texte) throws MauvaiseValeurDuJour, MauvaiseValeurDuMois, ValeurTropGrandeDuJour, MauvaisFormatDeDate {
    	if (texte.matches("^[0-9]{1,2}/[0-9]{1,2}/[0-9]{4}$")) {
			StringTokenizer st = new StringTokenizer(texte, "/");
			int j, m, a;
			j = Integer.parseInt(st.nextToken());
			m = Integer.parseInt(st.nextToken());
			a = Integer.parseInt(st.nextToken());
			initialise(j, m, a);
    		initialise(j, m, a);
    	} else {
    		throw new MauvaisFormatDeDate(texte);
    	}
    }

    public static String nomDuMois(int mois) {
    	switch (mois) {
	    	case  1: return "janvier";
	    	case  2: return "février";
	    	case  3: return "mars";
	    	case  4: return "avril";
	    	case  5: return "mai";
	    	case  6: return "juin";
	    	case  7: return "juillet";
	    	case  8: return "août";
	    	case  9: return "septembre";
	    	case 10: return "octobre";
	    	case 11: return "novembre";
	    	case 12: return "décembre";
	    	default: return "";
    	}
    }
    /**
     * renvoie le jour du mois de la date
     *  
     * @return un entier qui précise le jour du mois
     */
    public int jour() {
        return leJour;
    }
    
    /**
     * renvoie le mois de la date
     * 
     * @return un entier qui précise le mois de la date
     */
    public int mois() {
    	return leMois;
    }
    
    /**
     * renvoie l'année de la date
     * 
     * @return un entier qui précise l'année de la date
     */
    public int annee() {
    	return lAnnee;
    }
    
    /**
     * méthode qui renvoie le nombre de jour d'un mois quelconque
     * 
     * @param mois un entier précisant le mois de l'année
     * @param annee un entier précisant l'année
     */
    public static int nombreDeJoursDuMois(int mois, int annee) {
        switch (mois) {
            case 1 :
            case 3 :
            case 5 :
            case 7 :
            case 8 :
            case 10:
            case 12:
                return 31;
            case 4 :
            case 6 :
            case 9 :
            case 11:
                return 30;
            case 2 :
                return bissextile(annee) ? 29 : 28;
            default:
                return 0;
        }
    }
    
    /**
     * détermine si une année est bissextile ou non
     * 
     * @param annee un entier précisant l'année
     * @return vrai si l'année compte 366 jours, faux si elle compte 365 jours
     */
    public static boolean bissextile(int annee) {
        if (annee % 4 != 0) return false;
        if (annee % 100 == 0) return annee % 400 == 0;
        return true;
    }

    /**
     * méthode qui fait passer la date au jour suivant
     */
    public void jourPlus1() {
        leJour++;
        if (leJour > nombreDeJoursDuMois(leMois, lAnnee)) {
            leJour = 1;
            moisPlus1();
        }
    }
    
    /**
     * méthode qui fait passer la date au mois suivant
     */
    public void moisPlus1() {
        leMois++;
        if (leMois > 12) {
            leMois = 1;
            lAnnee++;
        } else {
        	if (leMois == 2) {
        		int max = Date.nombreDeJoursDuMois(leMois, lAnnee);
        		if (leJour > max) leJour = max;
        	}
        }
    }

    /**
     * méthode qui fait passer la date à l'année suivante
     */
    public void anneePlus1() {
        lAnnee++;
        if (leMois == 2 && leJour == 29)
        	leJour = 28;
    }

    /**
     * méthode qui fait passer la date au jour précédant
     */
    public void jourMoins1() {
        leJour--;
        if (leJour < 1) {
            moisMoins1();
            leJour = nombreDeJoursDuMois(leMois, lAnnee);
        }
    }
    
    /**
     * méthode qui fait passer la date au mois précédant
     */
    public void moisMoins1() {
        leMois--;
        if (leMois < 1) {
            leMois = 12;
            anneeMoins1();
        } else {
        	int maxJours = nombreDeJoursDuMois(leMois, lAnnee);
        	if (leJour > maxJours) leJour = maxJours; 
        }
    }

    /**
     * méthode qui fait passer la date à l'année précédante
     */
    public void anneeMoins1() {
        lAnnee--;
        if (leMois == 2 && leJour == 29)
        	leJour = 28;
    }

    /**
     * renvoie le texte notant la date
     *
     * @return     le texte de la date au format jj/mm/aaaa
     */
    public String toString() {
        String txt;
        /* 
        if (j < 10) txt = "0";
        else txt = "";
        */
       txt = (leJour < 10) ? "0" : "";
       txt += leJour;
       txt += '/';
       if (leMois < 10) txt += "0";
       txt += leMois;
       txt += '/';
       txt += lAnnee;
       return txt;
    }
}

